# Generated by Django 2.0.3 on 2018-03-20 07:02

from django.db import migrations, models


class Migration(migrations.Migration):
	dependencies = [
		("djpaypal", "0002_payer_time_created"),
	]

	operations = [
		migrations.AddField(
			model_name="dispute",
			name="dispute_channel",
			field=models.CharField(blank=True, editable=False, max_length=8),
		),
		migrations.AddField(
			model_name="dispute",
			name="dispute_life_cycle_stage",
			field=models.CharField(blank=True, editable=False, max_length=15),
		),
	]
