# Generated by Django 3.0.1 on 2019-12-30 11:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djpaypal', '0011_disputes'),
    ]

    operations = [
        migrations.RunSQL("UPDATE djpaypal_billingagreement SET state = 'Cancelled' WHERE state = 'Canceled';")
    ]
