# Generated by Django 2.1.2 on 2018-11-18 15:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djpaypal', '0006_auto_20181001_1147'),
    ]

    operations = [
        migrations.AddField(
            model_name='sale',
            name='invoice_number',
            field=models.CharField(blank=True, help_text='Your own invoice or tracking ID number.', max_length=127),
        ),
    ]
