# Generated by Django 2.1.2 on 2019-11-10 15:20

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djpaypal', '0008_refund_refund_to_payer'),
    ]

    operations = [
        migrations.AlterField(
            model_name='billingagreement',
            name='shipping_address',
            field=django.contrib.postgres.fields.jsonb.JSONField(null=True),
        ),
    ]
